package com.malk.fengkaili.service.impl;

import com.malk.fengkaili.repository.dao.FKLDdContactDao;
import com.malk.fengkaili.repository.entity.FKLDdContactPo;
import com.malk.fengkaili.service.FKLService;
import com.malk.service.dingtalk.DDClient;
import com.malk.service.dingtalk.DDClient_Contacts;
import com.malk.utils.UtilList;
import com.malk.utils.UtilMap;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Predicate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class FKLImplService implements FKLService {


    @Autowired
    private DDClient ddClient;

    @Autowired
    private DDClient_Contacts ddClient_contacts;

    @Autowired
    private FKLDdContactDao fklDdContactDao;

    /**
     * 同步用户信息
     */
    @Override
    public void syncUserInfo() {
        // 匹配部门信息, 全量
        ddClient_contacts.getDepartmentId_all(ddClient.getAccessToken(), true).forEach(deptId -> {
            String deptName = ddClient_contacts.getDepartmentInfo(ddClient.getAccessToken(), deptId).get("name").toString();
            for (String userId : ddClient_contacts.listDepartmentUserId(ddClient.getAccessToken(), deptId)) {
                if (fklDdContactDao.existsByUserId(userId)) {
                    continue;
                }
                Map userinfo = ddClient_contacts.getUserInfoById(ddClient.getAccessToken(), userId);
                // 员工信息表, 落库
                fklDdContactDao.save(FKLDdContactPo.builder()
                        .userId(userId)
                        .name(UtilMap.getString(userinfo, "name"))
                        .jobNumber(UtilMap.getString(userinfo, "job_number"))
                        .deptId(deptId)
                        .deptName(deptName)
                        .mobile(UtilMap.getString(userinfo, "mobile"))
                        .build());
            }
        });
    }

    /**
     * 查询用户列表
     */
    @Override
    public Page<FKLDdContactPo> queryUserInfos(int page, int size, String name, List<Long> deptIds) {

        // 分页 & 排序
        Sort sort = Sort.by(Sort.Direction.DESC, "deptId");
        Pageable pageable = PageRequest.of(page - 1, size, sort);

        // 查询条件: 姓名, 所属部门
        Specification<FKLDdContactPo> specification = (root, criteriaQuery, criteriaBuilder) -> {
            List<Predicate> predicateList = new ArrayList<>();
            if (StringUtils.isNotBlank(name)) {
                predicateList.add(criteriaBuilder.equal(root.get("name"), name));
            }
            if (UtilList.isNotEmpty(deptIds)) {
                predicateList.add(criteriaBuilder.in(root.get("deptId")).value(deptIds));
            }
            return criteriaBuilder.and(predicateList.toArray(new javax.persistence.criteria.Predicate[predicateList.size()]));
        };
        // 无数据时返回空列表
        return fklDdContactDao.findAll(specification, pageable);
    }
}
